function [C1,P1,C2,P2] = Tomasi(W)

% This is the simple part of the '92 "Shape and Motion from Image Streams
% Under Orthography" paper of Tomasi and Kanade.

% Written by Steve Marschner, April 1997

% W is a 2n by m measurement matrix in which the top half contains the
%   measured x coordinates of n projected points in m images and the bottom
%   half contains the corresponding x coordinates.

% C will be a 2n by 3 matrix containing the (perpendicular) u and v basis
%   vectors for the cameras (all u first, then all v).
% P will by a 3 by m matrix containing the locations of all the points.
% I return two C's and two P's because there's an abiguity I can't resolve --
%   the caller needs to be able to look at them and make a decision as to
%   which is the right one.

   [twon, m] = size(W);
   n = twon / 2;

   top = 1:n;
   bot = n+1:2*n;


   % First, use SVD to factor W.  Since we expect W to be rank 3, we can
   % factor it into a 2n by 3 and a 3 by m.  SVD gives us an approximation of
   % this in the case W is only near rank 3.

   [U,S,V] = svd(W,0);

   C = U(:,1:3);
   P = S(1:3,1:3)*V(:,1:3)';


   % Now we have the results in an arbitrary, non-orthonormal coordinate
   % system that the SVD computation chose.  Let's square it up using the
   % knowledge that the rows of C should be unit length and pairwise
   % orthogonal.
  
   % We'll do this by solving the equations
   %
   %     u_i X X' u_i = 1  (1)
   %     v_i X X' v_i = 1  (2) 
   %     u_i X X' v_i = 0  (3)
   %
   % These constitute a set of linear equations in (X X').

   % Each row of M represents one of these equations -- the first n are the n
   % instances of (1), the second n of (2), and the last n of (3).  The
   % columns of n correspond to the entries in the symmetric matrix (X X'),
   % ordered by diagonal.

   M = [  C(:,1).*C(:,1)   C(:,2).*C(:,2)   C(:,3).*C(:,3) ...
        2*C(:,1).*C(:,2) 2*C(:,2).*C(:,3) 2*C(:,1).*C(:,3);

          C(top,1).*C(bot,1) C(top,2).*C(bot,2) C(top,3).*C(bot,3) ...
          C(top,1).*C(bot,2) + C(top,2).*C(bot,1) ...
          C(top,2).*C(bot,3) + C(top,3).*C(bot,2) ...
          C(top,1).*C(bot,3) + C(top,3).*C(bot,1)];

   x = M \ [ones(2*n,1); zeros(n,1)];

   XXt = [x([1 4 6])'
          x([4 2 5])'
          x([6 5 3])'];


   % Now we have (X X'); Cholesky can give us a plausible X.

   X = chol(XXt)';

   C = C * X;
   P = X \ P;



   % If all is perfect, C will have the orthogonality and normality conditions
   % described above.  However, in the presence of error it won't.  Let's
   % enforce those conditions by adjusting C to make it right.

   % First, make all the rows into unit vectors.

   C = diag(1./sqrt(sum(C'.^2))) * C;

   % Do itty bitty SVDs to get to nearby orthonormal pairs.

   for k = 1:n
      [U,S,V] = svd([C(k+[0 n],:)'], 0);
      C(k+[0 n],:) = (U*V')';
   end;


   % There's still an orthogonal transform left, since X Q Q' X' == X X'.
   % We'll choose that so that the first camera is [1 0 0; 0 1 0].

   % Extend transpose of first camera to an ONB and that's the transform.
   % Unfortunately there are two ways to do that, corresponding to our
   % arbitrary orthogonal coordinate system being left-handed or
   % right-handed.  I'll return both, and the caller will have to figure out
   % which one is right-handed.

   Q = C([1 n+1],:)';
   Q = [Q cross(Q(:,1), Q(:,2))];
   C1 = C * Q;
   P1 = Q' * P;
   Q = Q * diag([1 1 -1]);
   C2 = C * Q;
   P2 = Q' * P;



